import { getSessionAction } from "lib/session";
import Link from "next/link";

import {
    ChangeEmailToken,
    verifyChangeEmailToken,
} from "@snip/auth/tokens/changeEmail";
import service from "@snip/database";

export default async function VerifyPage({
    searchParams,
}: {
    searchParams: { [key: string]: string | string[] | undefined };
}) {
    const session = await getSessionAction();
    const user = await service.user.getById(session!.user!.id);
    const token = searchParams["token"] as string | undefined;

    let tokenData: ChangeEmailToken | undefined;
    if (token) {
        try {
            tokenData = await verifyChangeEmailToken(user, token);
        } catch (error) {
            tokenData = undefined;
        }
    }

    if (!tokenData) {
        return (
            <div className="d-flex flex-grow-1 justify-content-center align-items-center flex-column">
                <h1>No token provided or invalid token</h1>
                <p>
                    If you want to request an email change, please go to your
                    profile settings and request a new email change.
                </p>
                <Link href="/profile">Go back to profile</Link>
            </div>
        );
    }

    // Request email change
    user.email = tokenData.to_email;

    await service.user.update(user);

    return (
        <div className="d-flex flex-grow-1 justify-content-center align-items-center flex-column">
            <h1>Email changed</h1>
            <p>
                Successfully changed email from {tokenData.from_email} to{" "}
                {tokenData.to_email}
            </p>
            <p>Make sure to use your new email for future logins.</p>
        </div>
    );
    //Change email
}
