"use client";

import { ButtonHTMLAttributes, useState } from "react";
import { Button, ButtonProps } from "react-bootstrap";
import { useFormStatus } from "react-dom";

export function SubmitButton({
    children,
    pending,
    ...props
}: ButtonHTMLAttributes<HTMLButtonElement> & { pending?: boolean }) {
    const { pending: form_pending } = useFormStatus();

    // Some reasonable defaults
    if (!props.className) {
        props.className = "btn btn-primary m-1";
    }
    if (!props.type) {
        props.type = "submit";
    }

    return (
        <button {...props} disabled={form_pending || pending || props.disabled}>
            {form_pending || pending ? (
                <>
                    <span
                        className="spinner-grow spinner-grow-sm"
                        role="status"
                        aria-hidden="true"
                    ></span>
                </>
            ) : (
                children
            )}
        </button>
    );
}

export function SecondaryButton({
    children,
    ...props
}: ButtonHTMLAttributes<HTMLButtonElement>) {
    if (!props.className) {
        props.className = "btn btn-outline-dark m-1";
    }
    if (!props.type) {
        props.type = "button";
    }

    return <button {...props}>{children}</button>;
}

export function ButtonWithConfirm({
    onConfirmed,
    children,
    ...props
}: ButtonProps & {
    onConfirmed: () => void;
    children: React.ReactNode;
}) {
    const [confirmed, setConfirmed] = useState(false);

    const handleConfirm = () => {
        if (confirmed) {
            onConfirmed();
            setConfirmed(false);
        } else {
            setConfirmed(true);
        }
    };

    return (
        <>
            {confirmed ? (
                <Button variant="danger" {...props} onClick={handleConfirm}>
                    Sure?
                </Button>
            ) : (
                <Button
                    variant="outline-danger"
                    {...props}
                    onClick={handleConfirm}
                >
                    {children}
                </Button>
            )}
        </>
    );
}
