import { BaseData, BaseSnip, BaseSnipArgs, BaseView, SnipData } from "./base";

export interface PlaceholderData extends BaseData {
    originalType: string;
    height: number;
    width: number;
}

export interface PlaceholderSnipArgs extends BaseSnipArgs {
    originalType: string;
    width: number;
    height: number;
}

/** A placeholder snip can be used
 * in places where it might be too expensive to transfer a full
 * snip. This was mainly introduced to streamline the event
 * protocol.
 *
 * For instance instead of transferring image data we just transfer
 * a snip with the height, width precomputed.
 *
 */
export class PlaceholderSnip extends BaseSnip {
    type: string = "placeholder";

    height: number;
    width: number;
    originalType: string;

    constructor({
        originalType,
        height,
        width,
        ...baseArgs
    }: PlaceholderSnipArgs) {
        super({ ...baseArgs });
        this.width = width;
        this.height = height;
        this.originalType = originalType;
    }

    static from_data(
        data: SnipData<PlaceholderData, BaseView>,
    ): PlaceholderSnip {
        return new PlaceholderSnip({
            // placeholder
            originalType: data.data.originalType,
            height: data.data.height,
            width: data.data.width,
            // Base
            x: data.view?.x,
            y: data.view?.y,
            rot: data.view?.rot,
            mirror: data.view?.mirror,
            id: data.id,
            page_id: data.page_id,
            book_id: data.book_id,
            last_updated: data.last_updated,
            created: data.created,
        });
    }

    static from_snip(snip: BaseSnip): PlaceholderSnip {
        const data = snip.to_data() as SnipData<PlaceholderData, BaseView>;
        data.data.width = snip.width;
        data.data.height = snip.height;
        data.data.originalType = snip.type;

        return PlaceholderSnip.from_data(data);
    }

    public render(): void {
        throw new Error("Method not implemented.");
    }
}
